from radicale.syno.singleton import Singleton
import cachetools.func
import radicale.syno.constant as constant
import ctypes


class SynoLib(metaclass=Singleton):
    def __init__(self):
        self._lib = ctypes.cdll.LoadLibrary(constant.SYNO_LIB_PATH)

    @cachetools.func.ttl_cache(maxsize=128, ttl=30)
    def is_authenticated(self, user, password, remote_host):
        return_code = self._lib.IsAuthenticated(
            str.encode(user),
            str.encode(password),
            # NOTE:
            # 1. remote_host contains extra string when HTTP_X_FORWARDED_FOR is set
            # ex: remote_host would be "'11.22.33.44' (forwarded by 55.66.77.88)"
            # (see radicale's __init__.py)
            # 2. HTTP_X_FORWARDED_FOR may contain more than one ip, which are seperated by ","
            # ref. https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Forwarded-For
            #
            # so we need to process "'11.22.33.44, 55.66.77.88' (forwarded by 55.66.77.88)" into 11.22.33.44
            str.encode(remote_host.split(" ")[0].strip("'").strip(","))
        )
        return (return_code == 0)

    @cachetools.func.ttl_cache(maxsize=128, ttl=30)
    def get_real_name(self, user_name):
        buf = ctypes.create_string_buffer(constant.SYNO_MAX_USERNAME_LENGTH)
        self._lib.GetRealName(
            str.encode(user_name),
            buf,
            constant.SYNO_MAX_USERNAME_LENGTH
        )
        return buf.value.decode("utf-8")
